'use strict';

/**
 * Service spécifique à l'application Sauvons nos tombes
 * Permet de faire les requêtes plus facilement.
 **/
var generateSqlService = [
    'GeneanetDate',
    'GeneanetDatabase',
    function(GeneanetDate, GeneanetDatabase) {
        var ORIENTATION_PORTRAIT = "portrait";
        var ORIENTATION_LANDSCAPE = "landscape";

        /**
         * Permet d'insérer un projet dans la Base de données (il faut après utiliser transaction)
         *
         * @return {object} Retourne un object contenant la requete et les params qui lui sont liés.
         */
        this.insertionProjet = function(id_projet_gnn, nom_projet, place, type_projet, geo, proprietaire) {
            var moment = GeneanetDate.getDateCurrent();

            var request = 'INSERT INTO projet (id_projet_gnn, nom_projet, place, type_projet, date_creation, is_sync, geo, proprietaire) VALUES (?, ?, ?, ?, ?, ?, ?, ?)';
            var params = [id_projet_gnn, nom_projet, place, type_projet, moment, false, geo, proprietaire];

            var requestObject = GeneanetDatabase.createRequest(request, params);
            requestObject.setIsReadAndWrite();

            return requestObject;
        }

        /**
         * Permet d'insérer un dépot dans la Base de données (il faut après utiliser transaction)
         *
         * @return {object} Retourne un object contenant la requete et les params qui lui sont liés.
         */
        this.insertionDepot = function(id_projet_local, lieu_depot, base64_portrait, base64_landscape) {
            if (base64_portrait == undefined) {
                base64_portrait = null;
            }

            if (base64_landscape == undefined) {
                base64_landscape = null;
            }
            var request = 'INSERT INTO depot (id_projet_local,lieu_depot, base64_portrait, base64_landscape) VALUES (?, ?, ?, ?)';
            var params = [id_projet_local, lieu_depot, base64_portrait, base64_landscape];

            var requestObject = GeneanetDatabase.createRequest(request, params);
            requestObject.setIsReadAndWrite();

            return requestObject;
        }


        /**
         * Permet d'insérer une photo dans la Base de données (il faut après utiliser transaction)
         *
         * @return {object} Retourne un object contenant la requete et les params qui lui sont liés.
         */
        this.insertionPhoto = function(native_url, id_depot, is_sync) {
            var request = 'INSERT INTO photo (native_url, id_depot, is_sync) VALUES (?, ?, ?)';
            var params = [native_url, id_depot, is_sync];

            var requestObject = GeneanetDatabase.createRequest(request, params);
            requestObject.setIsReadAndWrite();

            return requestObject;
        }

        /**
         * Permet d'insérér une variable de configuration.
         * 
         * @param {string}         key   Nom de la variable de configuration.
         * @param {string|integer} value Valeur de la configuration.
         * 
         * @return {object} Retourne un object contenant la requete et les params qui lui sont liés.
         */
        this.insertionConfiguration = function(key, value) {
            var request = 'INSERT INTO configuration (key, value) VALUES (?, ?)';
            var params = [key, value];

            var requestObject = GeneanetDatabase.createRequest(request, params);
            requestObject.setIsReadAndWrite();

            return requestObject;
        }

        /**
         * Permet de mettre à jour une configuration.
         * 
         * @param {string}         key   Nom de la variable de configuration.
         * @param {string|integer} value Valeur de la configuration.
         * 
         * @return {object} Retourne un object contenant la requete et les params qui lui sont liés.
         */
        this.updateConfiguration = function(key, value) {
            var request = 'UPDATE configuration SET value = ? WHERE key = ?';
            var params = [value, key];

            var requestObject = GeneanetDatabase.createRequest(request, params);
            requestObject.setIsReadAndWrite();

            return requestObject;
        }

        /**
         * Permet de supprimer une liste de variables de configurations.
         * 
         * @param {integer|array} keys Liste des variables de conf a supprimer.
         * 
         * @return {object} Retourne un object contenant la requete et les params qui lui sont liés.
         */
        this.deleteConfiguration = function(keys) {
            var request = "DELETE FROM configuration WHERE key IN " + GeneanetDatabase.generateClauseWhereIn(keys);

            var requestObject = GeneanetDatabase.createRequest(request);
            requestObject.setIsReadAndWrite();

            return requestObject;
        }

        /**
         * Permet de mettre à jour certaines infos sur le projet.
         * 
         * @param {string} idProjectLocal Id local du projet que l'on veut modifier.
         * @param {string} nameProject    Nouveau nom.
         * @param {string} placeProject   Nouvelle ville.
         * @param {string} typeProject    Nouveau type de projet.
         * 
         * @return {object} Retourne un object contenant la requete et les params qui lui sont liés.
         */
        this.updateProject = function(idProjectLocal, nameProject, placeProject, typeProject) {
            var request = "UPDATE projet SET nom_projet = ?, place = ?, type_projet = ?, is_sync = 'false' WHERE id_projet_local = ?";
            var params = [nameProject, placeProject, typeProject, idProjectLocal];

            var requestObject = GeneanetDatabase.createRequest(request, params);
            requestObject.setIsReadAndWrite();

            return requestObject;
        }

        /**
         * Permet de mettre à jour la localisation d'un projet.
         * 
         * @param {integer} idProjectLocal Id du projet en local.
         * @param {float}   lat            Nouvelle latitude.
         * @param {float}   lon            Nouvelle longitude.
         * 
         * @return {object} Retourne un object contenant la requete et les params qui lui sont liés.
         */
        this.updateProjectLocalisation = function(idProjectLocal, lat, lon) {
            var geo = lat+","+lon;
            var request = "UPDATE projet SET geo = ?, is_sync = 'false' WHERE id_projet_local = ?";
            var params = [geo, idProjectLocal];

            var requestObject = GeneanetDatabase.createRequest(request, params);
            requestObject.setIsReadAndWrite();

            return requestObject;
        }

        /**
         * Permet de mettre à jour une liste de photos depuis leur ids pour les passées en synchronisées.
         * 
         * @param {integer|array} idsPictures ids des photos à mettre comme synchronisées.
         * 
         * @return {object} Retourne un object contenant la requete et les params qui lui sont liés.
         */
        this.updatePictureIsSync = function(idsPictures) {
            var request = "UPDATE photo SET is_sync = 'true' WHERE id_picture IN " + GeneanetDatabase.generateClauseWhereIn(idsPictures);
            var params = [];

            var requestObject = GeneanetDatabase.createRequest(request, params);
            requestObject.setIsReadAndWrite();

            return requestObject;
        }

        /**
         * Permet de mettre un projet à jour après qu'il est été synchronisé et le taggué comme synchronisé.
         * 
         * @param {integer} idProjectLocal Id du projet côté application.
         * @param {integer} idProjectGnn   Id du projet côté GNN.
         * @param {string}  proprietaire   Sourcename du propriétaire s'il y en a un.
         * 
         * @return {object} Retourne un object contenant la requete et les params qui lui sont liés.
         */
        this.updateProjectSynchronized = function(idProjectLocal, idProjectGnn, proprietaire) {
            var request = "UPDATE projet SET id_projet_gnn = ?, proprietaire = ?, is_sync = 'true' WHERE id_projet_local = ?";
            var params = [idProjectGnn, proprietaire, idProjectLocal];

            var requestObject = GeneanetDatabase.createRequest(request, params);
            requestObject.setIsReadAndWrite();

            return requestObject;
        }

        /**
         * Permet de mettre à jour le dépot d'images depuis leurs ids.
         * 
         * @param {integer} idsPictures  ids des images.
         * @param {integer} newIdDepot id du dépot.
         * 
         * @return {object} Retourne un object contenant la requete et les params qui lui sont liés.
         */
        this.updateDepotPictures = function(idsPictures, newIdDepot) {
            var request = "UPDATE photo SET id_depot = ? WHERE id_picture IN " + GeneanetDatabase.generateClauseWhereIn(idsPictures);
            var params = [newIdDepot];

            var requestObject = GeneanetDatabase.createRequest(request, params);
            requestObject.setIsReadAndWrite();

            return requestObject;
        }

        /**
         * Permet de mettre à jour en BDD le code en base64 de la miniature du dépôt en mode portrait.
         * 
         * @param  {integer} idDepot Id du depot visé.
         * @param  {string}  base64  Base 64 de l'image miniaturisée.
         * 
         * @return {object} Retourne un object contenant la requete et les params qui lui sont liés.
         */
        this.updateDepotBase64Portrait = function(idDepot, base64) {
            return updateDepotBase64(idDepot, base64, ORIENTATION_PORTRAIT);
        }

        /**
         * Permet de mettre à jour en BDD le code en base64 de la miniature du dépôt en mode paysage.
         * 
         * @param  {integer} idDepot Id du depot visé.
         * @param  {string}  base64  Base 64 de l'image miniaturisée.
         * 
         * @return {object} Retourne un object contenant la requete et les params qui lui sont liés.
         */
        this.updateDepotBase64Landscape = function(idDepot, base64) {
            return updateDepotBase64(idDepot, base64, ORIENTATION_LANDSCAPE);
        }

        /**
         * Permet de vider en base de données les miniatures générées pour un dépôt donné.
         * 
         * @param  {integer} idDepot Id du depot visé.
         * 
         * @return {object} Retourne un object contenant la requete et les params qui lui sont liés.
         */
        this.resetBase64 = function(idDepot) {
            var request = "UPDATE depot SET base64_portrait = '', base64_landscape = '' WHERE id_depot = ?";
            var params = [idDepot];

            var requestObject = GeneanetDatabase.createRequest(request, params);
            requestObject.setIsReadAndWrite();

            return requestObject;
        }

        /**
         * Permet de mettre à jour la miniature d'un depot en précisant une orientation.
         * 
         * @param  {integer} idDepot     Id du dépot visé.
         * @param  {string}  base64      Base 64 de l'image miniaturisée.
         * @param  {string}  orientation Orientation de la miniature.
         * 
         * @return {object} Retourne un object contenant la requete et les params qui lui sont liés.
         */
        var updateDepotBase64 = function(idDepot, base64, orientation) {
            var request = "UPDATE depot SET ";
            if (orientation == ORIENTATION_PORTRAIT) {
                request += "base64_portrait = ?";
            } else {
                request += "base64_landscape = ?";
            }
            request += " WHERE id_depot = ?";
            var params = [base64, idDepot];

            var requestObject = GeneanetDatabase.createRequest(request, params);
            requestObject.setIsReadAndWrite();

            return requestObject;
        }

        /**
         * Permet de générer la requete de suppression en BDD d'une série de photos via leur id.
         * 
         * @param {array} ids Liste d'ids de photo.
         *
         * @return {object} Retourne un object contenant la requete et les params qui lui sont liés.
         */
        this.deletePhotosFromIds = function(ids) {
            var request = "DELETE FROM photo WHERE id_picture IN " + GeneanetDatabase.generateClauseWhereIn(ids);

            var requestObject = GeneanetDatabase.createRequest(request);
            requestObject.setIsReadAndWrite();

            return requestObject;
        }

        /**
         * Permet de générer la requete de suppression en BDD d'une série de depots via leur id.
         * 
         * @param {array} ids Liste d'ids de depots.
         *
         * @return {object} Retourne un object contenant la requete et les params qui lui sont liés.
         */
        this.deleteDepotsFromIds = function(ids) {
            var request = "DELETE FROM depot WHERE id_depot IN " + GeneanetDatabase.generateClauseWhereIn(ids);

            var requestObject = GeneanetDatabase.createRequest(request);
            requestObject.setIsReadAndWrite();

            return requestObject;
        }

        /**
         * Permet de générer la requete de suppression en BDD d'une série de projects via leur id.
         * 
         * @param {array} ids Liste d'ids de projects.
         *
         * @return {object} Retourne un object contenant la requete et les params qui lui sont liés.
         */
        this.deleteProjectsFromIds = function(ids) {
            var request = "DELETE FROM projet WHERE id_projet_local IN " + GeneanetDatabase.generateClauseWhereIn(ids);

            var requestObject = GeneanetDatabase.createRequest(request);
            requestObject.setIsReadAndWrite();

            return requestObject;
        }

        /**
         * Permet de vider la table des depots vides.
         * 
         * @return {object} Return request object.
         */
        this.deleteEmptyDepot = function() {
            var request = "DELETE FROM depot WHERE (SELECT count(*) FROM photo AS p WHERE p.id_depot = depot.id_depot) = 0";

            var requestObject = GeneanetDatabase.createRequest(request);
            requestObject.setIsReadAndWrite();

            return requestObject;
        }

        /**
         * Permet d'obtenir le nombre de photos liées à un dépot.
         * 
         * @param  {integer} idDepot Id du dépot dont on cherche le nombre de photo liée.
         * 
         * @return {object} Retourne un object contenant la requete et les params qui lui sont liés.
         */
        this.getNbPhotosFromIdDepot = function(idDepot) {
            var request = "SELECT COUNT(*) AS nb_photos FROM photo WHERE id_depot = ?";
            var params = [idDepot];

            return GeneanetDatabase.createRequest(request, params);
        }

        /**
         * Permet d'obtenir les infos d'un depot et des images qui lui sont associées.
         * 
         * @param {array} ids List d'ids de depots.
         * 
         * @return {object} Retourne un object contenant la requete et les params qui lui sont liés.
         */
        this.getInfosPicturesFromIdDepots = function(ids) {
            var request = "SELECT * FROM depot AS d INNER JOIN photo AS p ON d.id_depot = p.id_depot WHERE d.id_depot IN " + GeneanetDatabase.generateClauseWhereIn(ids);

            return GeneanetDatabase.createRequest(request);
        }

        /**
         * Permet d'obtenir les infos des depots d'un projet avec quelques infos des images dedans pour la gallerie.
         * 
         * @param {integer}  idProject Id du projet.
         * @param {integer}  startLimit Débutter à partir d'un certain depot.
         * @param {integer}  maxLimit Limiter à un certain nombre de depots.
         * @param {boolean}  reverseOrder Normal order by or reverse.
         * 
         * @return {object} Retourne un object contenant la requete et les params qui lui sont liés.
         */
        this.getInfoDepotsFromProject = function(idProject, startLimit, maxLimit, reverseOrder) {
            var orderBy = "ASC";
            if (reverseOrder) {
                orderBy = "DESC";
            }

            var request = 'SELECT count(*) as nb_image, d.id_depot, p.native_url, p.id_picture, d.base64_portrait, d.base64_landscape FROM photo as p INNER JOIN depot as d ON p.id_depot = d.id_depot WHERE d.id_projet_local = ? GROUP BY p.id_depot ORDER BY p.id_depot '+orderBy;
            var params = [idProject];

            if (typeof startLimit != "undefined" && startLimit == parseInt(startLimit)) {
                request += " LIMIT ?";
                params.push(Math.abs(startLimit));
                if (typeof maxLimit != "undefined" && maxLimit == parseInt(maxLimit)) {
                    request += ",?";
                    params.push(Math.abs(maxLimit));
                }
            }

            return GeneanetDatabase.createRequest(request, params);
        }

        /**
         * Permet d'obtenir les infos sur le projet pour le supprimer de l'appli.
         * 
         * @param {integer} idProject Id du projet.
         * 
         * @return {object} Retourne un object contenant la requete et les params qui lui sont liés.
         */
        this.getInfosProjectToDelete = function(idProject) {
            var request = "SELECT d.id_depot, p.id_picture, p.native_url, p.is_sync FROM depot AS d INNER JOIN photo AS p ON d.id_depot = p.id_depot WHERE d.id_projet_local = ?";
            var params = [idProject];

            return GeneanetDatabase.createRequest(request, params);
        }

        /**
         * Permet de compter le nombre de photos des projets de l'application.
         * 
         * @return {object} Retourne un object contenant la requete et les params qui lui sont liés.
         */
        this.countPicturesFromAllProjects = function() {
            var request = "SELECT count(*) as nb_image, d.id_projet_local FROM photo as p INNER JOIN depot as d ON p.id_depot = d.id_depot GROUP BY d.id_projet_local";

            return GeneanetDatabase.createRequest(request);
        }

        /**
         * Permet de récupérer les projets avec leurs nombres de photos associées.
         * 
         * @param  {integer|array} idsProject Ids des projets dont on veut les informations.
         *
         * @return {object} Retourne un object contenant la requete et les params qui lui sont liés.
         */
        this.getProjectsWithCountPictures = function(idsProject) {
            var request = "SELECT p.*, count(*) as count_picture FROM projet as p";
            request += " LEFT JOIN depot as d ON p.id_projet_local = d.id_projet_local";
            request += " LEFT JOIN photo as picture ON d.id_depot = picture.id_depot";

            if (idsProject) {
                request += " WHERE p.id_projet_local IN "+GeneanetDatabase.generateClauseWhereIn(idsProject);
            }

            request += " GROUP BY p.id_projet_local";

            return GeneanetDatabase.createRequest(request);
        }

        /**
         * Récupérer les projets ayant besoin d'être synchronisés.
         * 
         * @param {integer|array} idsProject Liste d'ids pour filtrer. Optionnel.
         * 
         * @return {object} Retourne un object contenant la requete et les params qui lui sont liés.
         */
        this.getProjectsNeedSynchronize = function(idsProject) {
            var request = "SELECT p.*, count(*) as count_picture FROM projet as p";
            request += " INNER JOIN depot as d ON p.id_projet_local = d.id_projet_local";
            request += " INNER JOIN photo as picture ON d.id_depot = picture.id_depot";
            request += " WHERE picture.is_sync = 'false'";

            if (idsProject) {
                request += " AND p.id_projet_local IN "+GeneanetDatabase.generateClauseWhereIn(idsProject);
            }

            request += " GROUP BY p.id_projet_local ORDER BY p.date_creation DESC";

            return GeneanetDatabase.createRequest(request);
        }

        /**
         * Permet de récupérer la première image d'un projet depuis son ID.
         * 
         * @param {integer} idProject Id du projet visé.
         * 
         * @return {object} Retourne un object contenant la requete et les params qui lui sont liés.
         */
        this.getFirstPictureFromProject = function(idProject) {
            var request = "SELECT p.* FROM photo as p INNER JOIN depot as d ON p.id_depot = d.id_depot";
            request += " WHERE d.id_projet_local = ? LIMIT 0,1";
            var params = [idProject];

            return GeneanetDatabase.createRequest(request, params);
        }

        /**
         * Get all pictures in a project.
         * 
         * @param  {integer} idProject Project's id.
         * 
         * @return {object} Retourne un object contenant la requete et les params qui lui sont liés.
         */
        this.getPicturesFromProject = function(idProject) {
            var request = "SELECT p.* FROM photo as p INNER JOIN depot as d ON p.id_depot = d.id_depot";
            request += " WHERE d.id_projet_local = ?";
            var params = [idProject];

            return GeneanetDatabase.createRequest(request, params);
        }

        /**
         * Permet de récupérer les infos sur les images non synchronisées d'un projet.
         * 
         * @param  {integer} idProject ID du projet visé.
         * @param  {integer} offset    Offset du limit SQL.
         * @param  {integer} limit     Limit du limit SQL.
         * 
         * @return {object} Retourne un object contenant la requete et les params qui lui sont liés.
         */
        this.getPicturesNoSynchroByProject = function(idProject, offset, limit) {
            var request = "SELECT p.* FROM photo as p INNER JOIN depot as d ON p.id_depot = d.id_depot";
            request += " WHERE p.is_sync = 'false' AND d.id_projet_local = ?";
            var params = [idProject];

            if (!isNaN(offset)) {
                request += " LIMIT ?";
                params.push(offset);
                if (!isNaN(limit)) {
                    request += ", ?";
                    params.push(limit);
                }
            }

            return GeneanetDatabase.createRequest(request, params);
        }

        /**
         * Permet de récupérer la liste des photos déjà synchronisées.
         * 
         * @return {object} Retourne un object contenant la requete et les params qui lui sont liés.
         */
        this.getPicturesSynchronized = function() {
            var request = "SELECT * FROM photo WHERE is_sync = 'true'";
            var params = [];

            return GeneanetDatabase.createRequest(request, params);
        }

        /**
         * Permet de récupérer les informations sur des photos depuis la BDD.
         * 
         * @param  {object|integer} ids List d'ids de photos.
         * 
         * @return {object} Retourne un object contenant la requete et les params qui lui sont liés.
         */
        this.getPhotosFromIds = function(ids) {
            var request = "SELECT * FROM photo WHERE id_picture IN " + GeneanetDatabase.generateClauseWhereIn(ids);

            return GeneanetDatabase.createRequest(request);
        }

        /**
         * Permet de récupérer les informations sur des projets depuis la BDD.
         * 
         * @param  {object|integer} ids List d'ids de projets.
         * 
         * @return {object} Retourne un object contenant la requete et les params qui lui sont liés.
         */
        this.getProjectsFromIds = function(ids) {
            var request = "SELECT * FROM projet WHERE id_projet_local IN " + GeneanetDatabase.generateClauseWhereIn(ids);

            return GeneanetDatabase.createRequest(request);
        }

        /**
         * Permet de récupérer tous les projets.
         * 
         * @return {object} Retourne un object contenant la requete et les params qui lui sont liés.
         */
        this.getAllProjects = function() {
            return GeneanetDatabase.createRequest("SELECT * FROM projet");
        }

        /**
         * Permet de récupérer tous les projets triés par date de création DESC.
         *
         * @return {object} Retourne un object contenant la requete et les params qui lui sont liés.
         */
        this.getAllProjectsOrderByLastCreate = function() {
            return GeneanetDatabase.createRequest("SELECT * FROM projet ORDER BY date_creation DESC");
        }

        /**
         * Permet de récupérer toutes les configurations stockées dans la table.
         * 
         * @return {object} Retourne un object contenant la requete et les params qui lui sont liés.
         */
        this.getAllConfigurations = function() {
            return GeneanetDatabase.createRequest("SELECT * FROM configuration");
        }

        /**
         * Permet de récupérer une variable de configuration depuis sa clé.
         * 
         * @param  {string} key Clé de la variable ciblée.
         * 
         * @return {object} Retourne un object contenant la requete et les params qui lui sont liés.
         */
        this.getConfigurationsByKey = function(key) {
            var request = "SELECT * FROM configuration WHERE key = ?";
            var params = [key];

            return GeneanetDatabase.createRequest(request, params);
        }
    }
]