'use strict';

/**
 * Permet configurer le service GeneanetOverlay.
 */
angular.module('GeneanetService').provider('GeneanetOverlay',[
    function() {
        var backgroundColor = "black";
        
        /**
         * Permet de changer la couleur de fond de l'overlay.
         *
         * @param string newBackgroundColor
         */
        this.setBackgroundColor = function(newBackgroundColor) {
            backgroundColor = newBackgroundColor;
        }

        /**
         * Méthode appelé par angularjs pour charger le service.
         * 
         * @return object Retourne l'objet représentant le service.
         */
        this.$get = ["$rootScope", function($rootScope) {
            var GeneanetOverlay = new GeneanetOverlayPrototype($rootScope);
            GeneanetOverlay.setBackgroundColor(backgroundColor);

            return GeneanetOverlay;
        }]
    }
]);

/**
 * Service permettant de générer des overlay avec différents contenus.
 */
var GeneanetOverlayPrototype = function($rootScopeConstruct) {
    /**
     * Id CSS de l'overlay.
     *
     * @var string idOverlay
     */
    var idOverlay = "overlay";

    /**
     * Couleur du fond de l'overlay.
     *
     * @var string backgroundColor
     */
    var backgroundColor;

    /**
     * @var {integer} Position courante de la barre de progression.
     */
    var progressBarCurrent;

    /**
     * @var {integer} Taille maximale de la barre de progression.
     */
    var progressBarMax;

    /**
     * @var {object} rootScope pour dispatch event.
     */
    var $rootScope = $rootScopeConstruct;

    /**
     * @var {boolean} Indicate if overlay is closable or not.
     */
    var closable = false;

    /**
     * Permet de setter la couleur de background du fond.
     *
     * @param string newBackgroundColor
     */
    this.setBackgroundColor = function(newBackgroundColor) {
        backgroundColor = newBackgroundColor;
    }

    /**
     * Retourne la couleur actuelle pour le fond.
     *
     * @return string
     */
    this.getBackgroundColor = function() {
        return backgroundColor;
    }

    /**
     * Permet de générer l'html de l'overlay et l'ajouter dans la page.
     *
     * @param string title       Titre de l'overlay.
     * @param string description Description de l'overlay.
     */
    this.openOverlay = function(title, description) {
        var overlay = this.generateOverlay(title, description);
        open(overlay);
    }

    /**
     * Permet de générer l'html de l'overlay et l'ajouter dans la page.
     *
     * @param string description Description en dessous du spinner.
     */
    this.openSpinner = function(description) {
        var spinner = this.generateSpinner(description);
        open(spinner);
    }

    /**
     * Permet de générer l'html d'une progressBar et de l'ajouter dans la page.
     * 
     * @param  {string}  description Phrase placée en dessous de la barre de progression.
     * @param  {integer} start       Début de la barre de progression.
     * @param  {integer} max         Taille de la progressBar.
     */
    this.openProgressBar = function(description, start, max) {
        var progressBar = this.generateProgressBar(description, start, max);
        open(progressBar);
        updateProgressBar();
    }

    /**
     * Permet de générer l'html d'un overlay de dialog et de l'ajouter dans la page.
     *  
     * @param  {string} query  Question posée à l'utilisateur.
     * @param  {string} choise1Msg   Message du premier bouton.
     * @param  {string} choise1Class Classes CSS du premier bouton.
     * @param  {string} choise2Msg   Message du second bouton.
     * @param  {string} choise2Class Classes CSS du second bouton.
     */
    this.openDialog = function(query, choise1Msg, choise1Class, choise2Msg, choise2Class) {
        var dialog = this.generateDialog(query, choise1Msg, choise1Class, choise2Msg, choise2Class);
        open(dialog);
    }

    var open = function(element) {
        closeOverlay(true);
        angular.element("body > div").prepend(element);
    }

    /**
     * Indicate if overlay is open or not.
     * 
     * @return {Boolean} True if it is open, else false.
     */
    this.isOpen = function() {
        if (angular.element("#"+idOverlay).length >= 1) {
            return true;
        }

        return false;
    }

    /**
     * Indicate if overlay is closable or not.
     * 
     * @return {Boolean} True if it is closable, else false.
     */
    this.isClosable = function() {
        return closable;
    }

    /**
     * Permet de fermer l'overlay (quelque soit le type de l'overlay).
     */
    this.close = function() {
        closeOverlay();
    }

    /**
     * Permet de fermer l'overlay (quelque soit le type de l'overlay) depuis une autre méthode de l'objet.
     */
    var closeOverlay = function(noEvent) {
        angular.element("#"+idOverlay).remove();
        if (!noEvent) {
            $rootScope.$broadcast("GeneanetOverlayClose", {});
        }
    }

    /**
     * Permet de générer l'html de l'overlay.
     *
     * @param string title       Titre de l'overlay.
     * @param string description Description de l'overlay.
     *
     * @return object Retourne un objet représentant l'html de l'overlay.
     */
    this.generateOverlay = function (title, description) {
        closable = true;
        var minimalistOverlay = generateMinimalistOverlay();
        var content = generateContentOverlay(title, description);
        minimalistOverlay.append(content);

        return minimalistOverlay;
    }

    /**
     * Permet de générer l'html de l'overlay.
     *
     * @param string description Description en dessous du spinner.
     *
     * @return object Retourne un objet représentant l'html du spinner.
     */
    this.generateSpinner = function (description) {
        closable = false;
        var minimalistOverlay = generateMinimalistOverlay();
        var content = generateContentSpinner(description);
        minimalistOverlay.append(content);

        return minimalistOverlay;
    }

    /**
     * Permet de générer l'hml d'une progressBar.
     * 
     * @param  {string}  description Phrase placée en dessous de la barre de progression.
     * @param  {integer} start       Début de la barre de progression.
     * @param  {integer} max         Taille de la progressBar.
     */
    this.generateProgressBar = function(description, start, max) {
        progressBarCurrent = start;
        progressBarMax = max;
        closable = false;

        var minimalistOverlay = generateMinimalistOverlay();
        var content = generateContentProgessBar(description);
        minimalistOverlay.append(content);

        return minimalistOverlay;
    }

    /**
     * Permet de générer l'html d'un overlay de dialog
     *  
     * @param  {string} query  Question posée à l'utilisateur.
     * @param  {string} choise1Msg   Message du premier bouton.
     * @param  {string} choise1Class Classes CSS du premier bouton.
     * @param  {string} choise2Msg   Message du second bouton.
     * @param  {string} choise2Class Classes CSS du second bouton.
     */
    this.generateDialog = function(query, choise1Msg, choise1Class, choise2Msg, choise2Class) {
        closable = false;

        var minimalistOverlay = generateMinimalistOverlay();
        var content = generateContentDialog(query, choise1Msg, choise1Class, choise2Msg, choise2Class);
        minimalistOverlay.append(content);

        return minimalistOverlay;
    }

    /**
     * Permet de faire avancer la progressBar
     */
    this.advance = function() {
        progressBarCurrent++;

        if (this.progressBarIsFinished()) {
            progressBarCurrent = progressBarMax;
        }

        updateProgressBar();
    }


    /**
     * Permet de savoir si on a fini la progression de la barre.
     * 
     * @return {boolean} True => finished, False also.
     */
    this.progressBarIsFinished = function() {
        if (progressBarCurrent >= progressBarMax) {
            return true;
        } else {
            return false;
        }
    }

    /**
     * Permet de mettre à jour visuellement la progressBar.
     */
    var updateProgressBar = function() {
        // on met a jour le pourcentage
        var pourcent = Math.round(progressBarCurrent*100/progressBarMax);
        pourcent = pourcent.toString();
        if (pourcent.length < 2) {
            pourcent = "0"+pourcent;
        }
        pourcent = pourcent + "%";
        if (angular.element("#overlay-pourcentage")) {
            angular.element("#overlay-pourcentage").text(pourcent);
        }
        // on met a jour la barre
        if (angular.element("#overlay-progress-bar")) {
            angular.element("#overlay-progress-bar").css("width", pourcent);
        }
    }

    /**
     * Permet de générer l'html du content pour l'overlay.
     *
     * @param string title       Titre de l'overlay.
     * @param string description Description de l'overlay.
     *
     * @return object Retourne un objet représentant le contenu de l'overlay.
     */
    var generateContentOverlay = function(title, description) {
        var divContent = angular.element("<div>")
            .css("position", "absolute")
            .css("width", "75%")
            .css("top", "25%")
            .css("left", "12.5%")
            .css("max-height", "50%")
            .css("overflow-y", "auto")
            .css("box-shadow", "0px 0px 10px black");
        
        var headerBlock = angular.element("<div>")
            .css("min-height", "20px")
            .css("position", "relative")
            .css("background-color", "#efefef");

        var titleBlock = angular.element("<h3>")
            .css("margin", "0px")
            .css("padding", "5px")
            .css("text-align", "center")
            .css("color", "#396159")
            .html(title);

        var linkClose = angular.element("<a>")
            .css("cursor", "pointer")
            .css("position", "absolute")
            .css("top", "10px")
            .css("right", "10px")
            .attr("id", "close-overlay");
        
        linkClose.click(function(e){
            e.preventDefault();
            closeOverlay();
        });

        headerBlock.append(linkClose);
        headerBlock.append(titleBlock);
        divContent.append(headerBlock);

        var contentBlock = angular.element("<div>")
            .css("padding", "20px")
            .css("background-color", "white")
            .css("color", "#777777")
            .html(description);

        divContent.append(contentBlock);

        return divContent;
    }

    /**
     * Permet de générer l'html du content pour le spinner.
     *
     * @param string description Description de l'overlay.
     *
     * @return object Retourne un objet représentant le contenu de l'overlay.
     */
    var generateContentSpinner = function(description) {
        var divContent = angular.element("<div>")
            .css("position", "absolute")
            .css("width", "75%")
            .css("top", "25%")
            .css("left", "12.5%");

        var divText = angular.element("<div>")
            .css("color", "white")
            .css("text-align", "center")
            .css("margin-top", "20px")
            .html(description);

        var divSpinner = angular.element("<div>")
            .css("background-repeat", "no-repeat")
            .attr("id", "spinner-overlay");

        divContent.prepend(divSpinner);
        divContent.append(divText);

        return divContent;
    }

    /**
     * Permet de générer l'html du content pour la progressBar.
     * 
     * @param  {string} description Phrase placée en dessous de la barre de progression.
     * 
     * @return {object}             Retourne un objet représentant le contenu de la progressBar.
     */
    var generateContentProgessBar = function(description) {
        var divContent = angular.element("<div>")
            .css("position", "absolute")
            .css("width", "75%")
            .css("top", "25%")
            .css("left", "12.5%")
            .css("padding", "1rem")
            .css("background-color", "white")
            .css("box-shadow", "0px 0px 10px black");

        var divBar = angular.element("<div>")
            .css("border-radius", "5px")
            .css("background-color", "#d6d6d6")
            .css("margin-top", "3.7rem")
            .css("height", "1.5rem")
            .css("position", "relative");

        var divProgressBar = angular.element("<div>")
            .css("position", "absolute")
            .css("height", "100%")
            .css("background-color", "#62b013")
            .css("border-radius", "5px")
            .css("width", "15%")
            .attr("id", "overlay-progress-bar");

        var divTip = angular.element("<div>")
            .css("font-size", "1.5rem")
            .css("position", "absolute")
            .css("top", "-3.7rem")
            .css("color", "white")
            .css("background-color", "#389683")
            .css("padding", "5px")
            .css("right", "0px");

        var spanTip = angular.element("<span>")
            .text("")
            .attr("id", "overlay-pourcentage");

        var triangleTip = angular.element("<div>")
            .css("position", "absolute")
            .css("right", "0px")
            .css("bottom", "-0.7rem")
            .css("border-color", "transparent")
            .css("border-style", "solid")
            .css("border-left", "1rem solid transparent")
            .css("border-top", "0.7rem solid #389683")
            .css("border-bottom", "0px")
            .css("border-right", "0px");

        var divDescription = angular.element("<div>")
            .html(description);

        divTip.append(spanTip);
        divTip.append(triangleTip);

        divProgressBar.append(divTip);

        divBar.append(divProgressBar);

        divContent.append(divBar);
        divContent.append(divDescription);

        return divContent;
    }

    /**
     * Permet de générer l'html du content de l'oberlay de dialog
     *  
     * @param  {string} query  Question posée à l'utilisateur.
     * @param  {string} choise1Msg   Message du premier bouton.
     * @param  {string} choise1Class Classes CSS du premier bouton.
     * @param  {string} choise2Msg   Message du second bouton.
     * @param  {string} choise2Class Classes CSS du second bouton.
     */
    var generateContentDialog = function(query, choise1Msg, choise1Class, choise2Msg, choise2Class) {
        var divContent = angular.element("<div>")
            .css("position", "absolute")
            .css("width", "75%")
            .css("top", "25%")
            .css("left", "12.5%")
            .css("max-height", "50%")
            .css("overflow-y", "auto")
            .css("box-shadow", "0px 0px 10px black");

        var divQuery = angular.element("<div>")
            .css("margin-bottom", "14px")
            .html(query);

        var buttonChoise1 = angular.element("<button>")
            .addClass(choise1Class)
            .css("width", "48%")
            .css("white-space", "initial")
            .css("margin-right", "2%")
            .html(choise1Msg)
            .click(function() {
                closeOverlay(true);
                $rootScope.$broadcast("overlayDialogResponse", {response: true});
            });

        var buttonChoise2 = angular.element("<button>")
            .addClass(choise2Class)
            .css("width", "48%")
            .css("white-space", "initial")
            .css("margin-left", "2%")
            .html(choise2Msg)
            .click(function() {
                closeOverlay(true);
                $rootScope.$broadcast("overlayDialogResponse", {response: false});
            });

        var divButtons = angular.element("<div>")
            .append(buttonChoise1)
            .append(buttonChoise2);

        var contentBlock = angular.element("<div>")
            .css("padding", "20px")
            .css("background-color", "white")
            .css("color", "#777777");

        contentBlock.append(divQuery);
        contentBlock.append(divButtons);

        divContent.append(contentBlock);

        return divContent;
    }

    /**
     * Permet de générer le minimum requis pour l'overlay.
     *
     * @return object Retourne un objet représentant l'html pour le minimum requis pour l'overay.
     */
    var generateMinimalistOverlay = function () {
        var container = generateContainer();
        var background = generateBackground();
        container.append(background);

        return container;
    }

    /**
     * Permet de générer l'html du conteneur de l'overlay.
     *
     * @return object Retourne un objet représentant l'html du block.
     */
    var generateContainer = function() {
        var divContainer = angular.element("<div>")
            .css("height", "100%")
            .css("position", "fixed")
            .css("top", "0px")
            .css("left", "0px")
            .css("width", "100%")
            .css("z-index", "100000");

        divContainer.attr("id", idOverlay);

        if (closable) {
            divContainer.click(function(){
                closeOverlay();
            });
        }

        return divContainer;
    }

    /**
     * Permet de générer l'html du background de l'overlay.
     *
     * @return object Retourne un objet représentant l'html du background.
     */
    var generateBackground = function() {
        var divBackground = angular.element("<div>")
            .css("height", "100%")
            .css("width", "100%")
            .css("position", "absolute")
            .css("background", backgroundColor)
            .css("opacity", "0.5");

        return divBackground;
    }
}
