"use strict";

(function(angular) {
    /**
     * GeneanetCache's exception.
     * 
     * @param {string} message Message to describe the error.
     */
    function GeneanetCacheException(message) {
        this.message = message;
    }

    GeneanetCacheException.prototype.name = "GeneanetCacheException";

    /**
     * Object to storage a data in the cache.
     * 
     * @param {mixed} value      Value which we want to cache.
     * @param {date}  dateExpire Date to invalidate cache.
     */
    function GeneanetCacheContainer(value, dateExpire) {
        this.value = value;
        this.dateExpire = dateExpire;
    }

    /**
     * Test if this data is expired.
     * 
     * @return {Boolean} True if it's expired, else false.
     */
    GeneanetCacheContainer.prototype.isExpired = function() {
        return new Date(this.dateExpire) < new Date() ? true : false;
    };

    /**
     * To generate a GeneanetCacheContainer from object in storage.
     * 
     * @param {Object} obj
     * 
     * @return {GeneanetCacheContainer}
     */
    function generateGeneanetCacheContainerFromStorageObject(obj) {
        var returnObj = new GeneanetCacheContainer();
        for (var key in obj) {
            returnObj[key] = obj[key];
        }

        return returnObj;
    }

    /**
     * Object to manage the cache.
     * 
     * @param {Object}   _storage   Storage.
     * @param {Function} _exception Exception to the cache.
     */
    function GeneanetCache(_storage, _exception) {
        /**
         * Return storage used.
         * 
         * @return {Object}
         */
        this.getStorage = function() {
            return _storage;
        };

        /**
         * Return type of exception used.
         * 
         * @return {Function}
         */
        this.getException = function() {
            return _exception;
        };

        /**
         * Get namespace to storage.
         * 
         * @return {string}
         */
        this.getNamespace = function() {
            return "GeneanetCache";
        };
    }

    /**
     * To add a new data in cache.
     * 
     * @param {string} key   Key to identifier the data.
     * @param {mixed}  value The value cached.
     * @param {[type]} ttl   Time to expire the data in second.
     */
    GeneanetCache.prototype.put = function(key, value, ttl) {
        var dateExpire = new Date();
        dateExpire.setSeconds(dateExpire.getSeconds()+ttl);
        this.getStorage().setItem(key, new GeneanetCacheContainer(value, dateExpire), this.getNamespace());
    };

    /**
     * To access a data in cache if this data is not expired.
     * 
     * @param {string} key To identifier the data.
     * 
     * @return {mixed}
     * @throws {GeneanetCacheException} If this data is not cached or it expired.
     */
    GeneanetCache.prototype.get = function(key) {
        if (!this.isCached(key)) {
            throw new (this.getException())("Key is not cached.");
        }

        var container = generateGeneanetCacheContainerFromStorageObject(
            this.getStorage().getItem(key, this.getNamespace())
        );

        if (container.isExpired()) {
            throw new (this.getException())("Key is expired."); 
        }

        return container.value;
    };

    /**
     * To check if the data is expired or not.
     * 
     * @param {string} key To identifier the data.
     * 
     * @return {Boolean} True if it is expired, else false.
     */
    GeneanetCache.prototype.isExpired = function(key) {
        if (!this.isCached(key)) {
            return true;
        }

        var container = generateGeneanetCacheContainerFromStorageObject(
            this.getStorage().getItem(key, this.getNamespace())
        );

        return container.isExpired();
    };

    /**
     * To check if the key in th cache.
     * 
     * @param {string} key To identifier the data.
     * 
     * @return {Boolean} True if it is cached, else false.
     */
    GeneanetCache.prototype.isCached = function(key) {
        return this.getStorage().itemExist(key, this.getNamespace());
    };

    /**
     * To remove the data identifier by the key.
     * 
     * @param {string} key To identifier the data.
     * 
     * @return {Boolean} True if success, else false.
     */
    GeneanetCache.prototype.remove = function(key) {
        return this.getStorage().removeItem(key, this.getNamespace());
    };

    /**
     * Remove all cache.
     */
    GeneanetCache.prototype.removeAll = function() {
        this.getStorage().clear(this.getNamespace());
    };

    /**
     * Clean datas expired.
     * 
     * @return {integer} Return the number of deleted datas.
     */
    GeneanetCache.prototype.clean = function() {
        var items = this.getStorage().getItems(this.getNamespace());
        var countDelete = 0;
        for (var key in items) {
            var container = generateGeneanetCacheContainerFromStorageObject(items[key]);
            if (container.isExpired()) {
                this.remove(key);
                countDelete++;
            }
        }

        return countDelete;
    };

    /**
     * Return the cache' size.
     * 
     * @return {integer}
     */
    GeneanetCache.prototype.size = function() {
        return this.getStorage().length(this.getNamespace());
    };

    angular.module("GeneanetService").factory("GeneanetPersistentCache", [
        "GeneanetLocalStorage",
        function (GeneanetLocalStorage) {
            /**
             * GeneanetPersistentCache's exception.
             * 
             * @param {string} message Message to describe the error.
             */
            function GeneanetPersistentCacheException(message) {
                GeneanetCacheException.call(this, message);
            }
            GeneanetPersistentCacheException.prototype = Object.create(GeneanetCacheException.prototype, {
                name: {
                    value: "GeneanetPersistentCacheException"
                }
            });

            function GeneanetPersistentCache() {
                GeneanetCache.call(this, GeneanetLocalStorage, GeneanetPersistentCacheException);
            }
            GeneanetPersistentCache.prototype = Object.create(GeneanetCache.prototype);
        
            return new GeneanetPersistentCache();
        }
    ]);

    angular.module("GeneanetService").factory("GeneanetSessionCache", [
        "GeneanetSessionStorage",
        function (GeneanetSessionStorage) {
            /**
             * GeneanetSessionCache's exception.
             * 
             * @param {string} message Message to describe the error.
             */
            function GeneanetSessionCacheException(message) {
                GeneanetCacheException.call(this, message);
            }
            GeneanetSessionCacheException.prototype = Object.create(GeneanetCacheException.prototype, {
                name: {
                    value: "GeneanetSessionCacheException"
                }
            });

            function GeneanetSessionCache() {
                GeneanetCache.call(this, GeneanetSessionStorage, GeneanetSessionCacheException);
            }
            GeneanetSessionCache.prototype = Object.create(GeneanetCache.prototype);
        
            return new GeneanetSessionCache();
        }
    ]);
})(window.angular);