"use strict";

(function(angular) {
    function StorageTypeNotFoundException(message) {
        this.message = message;
        this.name = "StorageTypeNotFoundException";
    }
    /**
     * Constructor to Storage Object.
     * 
     * @param {string} _typeStorage A type to storage (localStorage / sessionStorage).
     */
    function StorageObj(_typeStorage) {
        var _namespace = null;

        if (!window.hasOwnProperty(_typeStorage)) {
            throw new StorageTypeNotFoundException("Storage : '"+_typeStorage+"' not found in window property.");
        }

        _typeStorage = window[_typeStorage];
        
        /**
         * Getter type storage.
         * 
         * @return {Object} Return an object to represent storage used.
         */
        this.getTypeStorage = function() {
            return _typeStorage;
        };

        /**
         * Getter current namespace.
         * 
         * @return {string}
         */
        this.getNamespace = function() {
            return _namespace;
        };

        /**
         * Setter current namespace.
         * 
         * @param {string} namespace New current namespace.
         */
        this.setNamespace = function(namespace) {
            _namespace = namespace;
        };
    }

    /**
     * To check if namespace exist.
     * 
     * @param {string} namespace Namespace to check.
     * 
     * @return {boolean} Return true if exist, else false.
     */
    StorageObj.prototype.namespaceExist = function(namespace) {
        namespace = namespace ? namespace : this.getNamespace();

        return this.getTypeStorage().getItem(namespace) === null ? false : true;
    };

    /**
     * To get all items in a namespace.
     * 
     * @param {string} namespace Namespace needed.
     * 
     * @return {Object} Return items in an object or an object empty if namespace isn't exist.
     */
    StorageObj.prototype.getItems = function(namespace) {
        namespace = namespace ? namespace : this.getNamespace();
        var values = this.getTypeStorage().getItem(namespace);

        return values === null ? {} : JSON.parse(values);
    };

    /**
     * Clear current storage.
     */
    StorageObj.prototype.clearAll = function() {
        this.getTypeStorage().clear();
    };

    /**
     * Clear a namespace.
     * 
     * @param {string} namespace Namespace to clear.
     */
    StorageObj.prototype.clear = function(namespace) {
        namespace = namespace ? namespace : this.getNamespace();
        this.getTypeStorage().removeItem(namespace);
    };

    /**
     * Get one item's value in a namespace.
     * 
     * @param {string} key       Key to access item.
     * @param {string} namespace Namespace to search item.
     * 
     * @return {mixed|undefined} Return the value or undefined.
     */
    StorageObj.prototype.getItem = function(key, namespace) {
        var items = this.getItems(namespace);

        return items.hasOwnProperty(key) ? items[key] : undefined;
    };

    /**
     * Check if one item exist in namespace.
     * 
     * @param {string} key       Key to access item.
     * @param {string} namespace Namespace to search item.
     * 
     * @return {boolean} Return true if exist, else false.
     */
    StorageObj.prototype.itemExist = function(key, namespace) {
        var items = this.getItems(namespace);

        return items.hasOwnProperty(key) ? true : false;
    };

    /**
     * Get one item's value in a namespace and return defaultValue if it isn't found.
     * 
     * @param {string} key          Key to access item.
     * @param {mixed}  defaultValue Default value for item if it isn't found.
     * @param {string} namespace    Namespace to search item.
     * 
     * @return {mixed} Return the value or default value.
     */
    StorageObj.prototype.getItemWithDefaultValue = function(key, defaultValue, namespace) {
        var item = this.getItem(key, namespace);

        return item === undefined ? defaultValue : item;
    };

    /**
     * Count items in namespace.
     * 
     * @param {string} namespace Namespace where count items.
     * 
     * @return {integer}
     */
    StorageObj.prototype.length = function(namespace) {
        return Object.keys(this.getItems(namespace)).length;
    };

    /**
     * To set an item in a namespace.
     * 
     * @param {string} key       Key to access the item.
     * @param {mixed}  value     Item.
     * @param {string} namespace Namespace to storage item.
     */
    StorageObj.prototype.setItem = function(key, value, namespace) {
        namespace = namespace ? namespace : this.getNamespace();
        
        var items = this.getItems(namespace);
        items[key] = value;

        this.getTypeStorage().setItem(namespace, JSON.stringify(items));
    };

    /**
     * Remove an item in namespace.
     * 
     * @param {string} key       Item's key to access.
     * @param {string} namespace Namespace to search item.
     * 
     * @return {boolean} Return true if delete successful, else false.
     */
    StorageObj.prototype.removeItem = function(key, namespace) {
        namespace = namespace ? namespace : this.getNamespace();

        var items = this.getItems(namespace);
        var deleteSucceessful = delete items[key];
        if (deleteSucceessful) {
            this.getTypeStorage().setItem(namespace, JSON.stringify(items));
        }

        return deleteSucceessful;
    };

    /**
     * Service localStorage
     */
    angular.module("GeneanetService").factory("GeneanetLocalStorage", [function () {
        return new StorageObj("localStorage");
    }]);

    /**
     * Service sessionStorage
     */
    angular.module("GeneanetService").factory("GeneanetSessionStorage", [function () {
        return new StorageObj("sessionStorage");
    }]);
})(window.angular);